import 'package:flutter/material.dart';
import 'package:marianum_mobile/api/mhsl/message/getMessages/getMessagesResponse.dart';
import 'package:syncfusion_flutter_pdfviewer/pdfviewer.dart';
import 'package:url_launcher/url_launcher.dart';

class MessageView extends StatefulWidget {
  final String basePath;
  final GetMessagesResponseObject message;
  const MessageView({Key? key, required this.basePath, required this.message}) : super(key: key);

  @override
  State<MessageView> createState() => _MessageViewState();
}

class _MessageViewState extends State<MessageView> {

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.message.name),
      ),
      body: SfPdfViewer.network(
        widget.basePath + widget.message.url,
        enableHyperlinkNavigation: true,
        onDocumentLoadFailed: (PdfDocumentLoadFailedDetails e) {
          Navigator.of(context).pop();
          showDialog(context: context, builder: (context) {
            return AlertDialog(
              title: const Text("Fehler beim öffnen"),
              content: Text("Dokument '${widget.message.name}' konnte nicht geladen werden:\n${e.description}"),
              actions: [
                TextButton(onPressed: () {
                  Navigator.of(context).pop();
                }, child: const Text("Ok"))
              ],
            );
          });
        },
        onHyperlinkClicked: (PdfHyperlinkClickedDetails e) {
          showDialog(
            context: context,
            builder: (context) {
              return AlertDialog(
                title: const Text("Link öffnen"),
                content: Text("Möchtest du den folgenden Link öffnen?\n${e.uri}"),
                actions: [
                  TextButton(onPressed: () {
                    Navigator.of(context).pop();
                  }, child: const Text("Abbrechen")),
                  TextButton(onPressed: () {
                    launchUrl(Uri.parse(e.uri), mode: LaunchMode.externalApplication);
                  }, child: const Text("Öffnen")),
                ],
              );
            },
          );
        },
      ),
    );
  }
}
