import 'dart:convert';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_linkify/flutter_linkify.dart';
import 'package:marianum_mobile/api/marianumcloud/talk/chat/richObjectStringProcessor.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher_string.dart';

import '../../../api/marianumcloud/talk/chat/getChatResponse.dart';

class ChatMessage {
  String originalMessage;
  Map<String, RichObjectString>? originalData;

  RichObjectString? file;
  String content = "";

  bool get containsFile => file != null;

  ChatMessage({required this.originalMessage, this.originalData}) {
    if(originalData?.containsKey("file") ?? false) {
      file = originalData?['file'];
      content = file?.name ?? "Datei";
    } else {
      content = RichObjectStringProcessor.parseToString(originalMessage, originalData);
    }
  }

  Future<Widget> getWidget() async {
    SharedPreferences preferences = await SharedPreferences.getInstance();

    if(file == null) {
      return SelectableLinkify(
        text: content,
        onOpen: onOpen,
      );
    }

    return CachedNetworkImage(
      errorWidget: (context, url, error) {
        return Column(
          children: [
            const Icon(Icons.image_not_supported_outlined, size: 35),
            Text("Keine Dateivorschau:\n${file!.name}", style: const TextStyle(fontWeight: FontWeight.bold))
          ],
        );
      },
      alignment: Alignment.center,
      placeholder: (context, url) {
        return const Padding(padding: EdgeInsets.all(10), child: CircularProgressIndicator());
      },
      fadeInDuration: const Duration(seconds: 1),
      imageUrl: "https://cloud.marianum-fulda.de/core/preview?fileId=${file!.id}&x=100&y=-1&a=1",
      httpHeaders: {
        "Authorization": "Basic ${base64.encode(utf8.encode("${preferences.getString("username")}:${preferences.getString("password")}"))}" // TODO move authentication
      },
    );
  }

  void onOpen(LinkableElement link) async {
    if(await canLaunchUrlString(link.url)) {
      await launchUrlString(link.url);
    } else {

    }
  }
}